@@ -3,15 +3,16 @@ using namespace Rcpp;
 
 // [[Rcpp::interfaces(r, cpp)]]
 
-//' Get the means/medians/variances of pillars of a 3d array
+//' Get the sums/means/medians/variances of pillars of a 3d array.
 //'
 //' For a 3-dimensional array \code{arr3d}, pillar \code{ij} is defined as
-//' \code{arr3d[i, j, ]}. These functions compute the mean, median and variance of each
-//' pillar.
+//' \code{arr3d[i, j, ]}. These functions compute the mean, median and variance
+//' of each pillar.
 //'
 //' @param arr3d A 3-dimensional array.
 //'
-//' @return A matrix where element \code{i,j} is equal to \code{mean(arr3d[i, j, ])},
+//' @return A matrix where element \code{i,j} is equal to
+//' \code{sum(arr3d[i, j, ])}, \code{mean(arr3d[i, j, ])},
 //' \code{median(arr3d[i, j, ])}, or \code{var(arr3d[i, j, ])}.
 //'
 //' @examples
@@ -22,6 +23,26 @@ using namespace Rcpp;
 //'
 //' @export
 // [[Rcpp::export]]
+NumericMatrix sum_pillars(NumericVector arr3d) {
+  IntegerVector dim = arr3d.attr("dim");
+  int n_pillars = dim[0] * dim[1];
+  int pillar_len = dim[2];
+  NumericMatrix sums(dim[0], dim[1]);
+  double sum_i;
+  NumericVector pillar_i(pillar_len);
+  for (int i = 0; i < n_pillars; i++) {
+    for (int j = 0; j < pillar_len; j++) {
+      pillar_i[j] = arr3d[i + j * n_pillars];
+    }
+    sum_i = sum(pillar_i);
+    sums(i % dim[0], i / dim[0]) = sum_i;
+  }
+  return sums;
+}
+
+//' @rdname sum_pillars
+//' @export
+// [[Rcpp::export]]
 NumericMatrix mean_pillars(NumericVector arr3d) {
   IntegerVector dim = arr3d.attr("dim");
   int n_pillars = dim[0] * dim[1];
@@ -39,7 +60,7 @@ NumericMatrix mean_pillars(NumericVector arr3d) {
   return means;
 }
 
-//' @rdname mean_pillars
+//' @rdname sum_pillars
 //' @export
 // [[Rcpp::export]]
 NumericMatrix var_pillars(NumericVector arr3d) {
@@ -59,7 +80,7 @@ NumericMatrix var_pillars(NumericVector arr3d) {
   return vars;
 }
 
-//' @rdname mean_pillars
+//' @rdname sum_pillars
 //' @export
 // [[Rcpp::export]]
 NumericMatrix median_pillars(NumericVector arr3d) {
