@@ -3,9 +3,15 @@ test_that("mean_stack_thresh works", {
   img <- imageData(readImage(system.file("extdata", "50.tif",
     package = "autothresholdr"), as.is = TRUE))
   img_thresh_mask <- mean_stack_thresh(img, "Otsu")
-  expect_equal(round(mean(img_thresh_mask, na.rm = TRUE), 4), 25.2892)
+  expect_equal(round(mean(img_thresh_mask, na.rm = TRUE), 2), 24.09)
+  img_thresh_mask <- mean_stack_thresh(img, 13.2)
+  expect_equal(round(mean(img_thresh_mask, na.rm = TRUE), 2), 24.09)
   img_thresh_mask <- med_stack_thresh(img, "Triangle")
   expect_equal(round(mean(img_thresh_mask, na.rm = TRUE), 3), 23.622)
+  img_thresh_mask <- med_stack_thresh(img + 2 ^ 32, "Triangle")
+  expect_equal(round(mean(img_thresh_mask, na.rm = TRUE), 3), 23.622 + 2 ^ 32)
+  img_thresh_mask <- mean_stack_thresh(img + 2 ^ 32, "Otsu")
+  expect_equal(round(mean(img_thresh_mask, na.rm = TRUE), 2), 24.09 + 2 ^ 32)
   const.arr <- array(2, dim = rep(2, 3))
   expect_error(mean_stack_thresh(const.arr, "tri"))
   expect_error(med_stack_thresh(const.arr, "tri"))
